import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="smilecoin",
    version="1.2.4",
    author="Nick Bucheleres",
    author_email="nick@smilecoin.us",
    description="Smile Coin Python SDK",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SmileCoinUS/smilecoin-sdk",
    project_urls={
        "Bug Tracker": "https://github.com/SmileCoinUS/smilecoin-sdk",
        "Documentation": "https://docs.smilecoin.us"
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    # packages=setuptools.find_packages(include=['smilecoin', 'smilecoin-sdk', 'smilecoin-sdk.*']),
    packages=['smilecoin'],
    install_requires=['requests'],
    python_requires=">=3.6")


from algosdk.encoding import future_msgpack_decode

import requests

import json

documentation_link = 'https://docs.smilecoin.us'

class API:
    def __init__(self):
        self.api_key = None
        self.smile_price = requests.get('https://api.smilecoin.us/api/v1/smile/price').json()['price']
        self.environ = 'testnet'

    def auth(self, api_key):
        self.api_key = api_key


    def verify_txn(self, amount, txn):
        try:
            txn = future_msgpack_decode(txn).transaction
            # TODO: look up intended recipient address based on self.api_key
            platform_address = ''

            receiver, txn_amount = txn.receiver, txn.amount / 1000000

            txn_amount /= self.smile_price()
            if platform_address != receiver or amount != txn_amount:
                return False
            return True
        except Exception:
            return False

    def receive(self, txn):
        try:
            if self.api_key is None:
                return {'message': 'You must authorize your request first!  See documentation {}/accept-payments#2.-create-transaction.'.format(documentation_link)}

            headers = {'api-key': self.api_key, 'Content-type': 'application/json'}

            if not isinstance(txn, dict): return {'message': 'You must pass a dictionary!'}

            if self.environ == 'mainnet': r = requests.post('https://api.smilecoin.us/api/v1/smile/receive', headers=headers, json=txn)
            else: r = requests.post('https://api.smilecoin.dev/api/v1/smile/receive', headers=headers, json=txn)

            response = json.loads(r.content)

            if r.status_code != 201:
                message = 'Could not complete receive because {}.  Visit {}/receive-payments#3.-send-transaction for help.'.format(response, documentation_link)
                return {'status_code': r.status_code, 'message': message}

            return {'status_code': r.status_code, 'usd_amount': response['usd_amount'], 'message': response['message']}
        except Exception as e:
            return {'status_code': 500, 'usd_amount': None, 'message': 'Receive method failed because {}'.format(e)}


    def send(self, txn):
        try:
            if self.api_key is None:
                return {'message': 'You must authorize your request first!  See documentation {}/accept-payments#2.-create-transaction.'.format(documentation_link)}

            headers = {'api-key': self.api_key, 'Content-type': 'application/json'}

            if not isinstance(txn, dict): return {'message': 'You must pass a dictionary!'}

            if self.environ == 'mainnet': r = requests.post('https://api.smilecoin.us/api/v1/smile/send', headers=headers, json=txn)
            else: r = requests.post('https://api.smilecoin.dev/api/v1/smile/send', headers=headers, json=txn)

            response = json.loads(r.content)

            if r.status_code != 201:
                message = 'Could not complete send because {}.  Visit {}/send-payments#4.-complete-transaction for help.'.format(response, documentation_link)
                return {'status_code': r.status_code, 'message': message}

            return {'status_code': r.status_code, 'usd_amount': response['usd_amount'], 'message': response['message']}

        except Exception as e:
            return {'status_code': 500, 'usd_amount': None, 'message': 'Send method failed because {}'.format(e)}